! ============================================================================
! "WMAP_Read_Map" reads a WMAP skymap FITS file. The only mandatory output is
! the Status code, and only the components requested will be extracted from
! the file.
!
! If an output array is requested (by being present in the argument list) but
! unassociated then this routine will allocate space for it.  This is why the
! arrays are all declared as pointers.
!
! This routine assumes that the programmer knows what is in the file.
! Therefore requesting something that isn't in the file will generate an
! error.
!
! This routine requires the FITSIO library, available from
! http://heasarc.gsfc.nasa.gov/docs/software/fitsio/fitsio.html
!
! Arguments:
!	File    - The name of the file.
!	Status  - A status code:  0=success.  Most of these are FITS status
!	          codes, though memory allocation errors are also passed back
!	          unchanged.
!	Temp    - The Stokes I/Temperature skymap.
!	QPol    - The Stokes Q skymap.
!	UPol    - The Stokes U skymap.
!	SpSig   - The bandpass mismatch/spurious signal skymap.
!	NObs    - The N_Obs array.
!	Covar   - The polarization covariance matrices.  This is either a
!	          2x2 or 3x3 matrix for each pixel, depending upon the 
!	          presence of a spurious signal map.
!	NPixels - The number of pixels in a map.
!
! Written by Michael R. Greason, SSAI, 13 March 2006.
! ============================================================================
Subroutine WMAP_Read_Map (File, Status, Temp, QPol, UPol, SpSig,	&
                          NObs, Covar, NPixels)
!
Implicit None
!
Character (*),                   Intent(In)            :: File
Integer (Kind=4),                Intent(Out)           :: Status
Real (Kind=4), Dimension(:),     Pointer,     Optional :: Temp
Real (Kind=4), Dimension(:),     Pointer,     Optional :: QPol
Real (Kind=4), Dimension(:),     Pointer,     Optional :: UPol
Real (Kind=4), Dimension(:),     Pointer,     Optional :: SpSig
Real (Kind=4), Dimension(:),     Pointer,     Optional :: NObs
Real (Kind=4), Dimension(:,:,:), Pointer,     Optional :: Covar
Integer (Kind=4),                Intent(Out), Optional :: NPixels
!
Integer (Kind=4) :: unit, lst, rwmode, tmp, npix
! ----------------------------------------------------------------------------
If (Present(NPixels)) NPixels = 0
Status  = 0
!
!			Open the FITS file.  Position it to the first
!			binary extension header/data unit (HDU).
!
rwmode = 0
Call FTGIOU (unit, Status)				! Get a free unit #.
Call FTOPEN (unit, File, rwmode, tmp, Status)		! Open the file.
If (Status .NE. 0) Return
!
Call FTMAHD (unit, 2, tmp, Status)			! First ext. HDU. 
If (Status .NE. 0) Return
!
!			How many pixels are in the map?  Get this by
!			getting the row count.
!
Call FTGNRW (unit, npix, Status)
If ((npix .LE. 0) .OR. (Status .NE. 0)) Go To 99
!
!			Extract data from this first extension table.
!
!				Get the Stokes I/Temperature columm.
!
If (Present(Temp)) Then
  Call FetchColumn (Unit, npix, 'TEMPERATURE', Temp, Status)
  If (Status .NE. 0) Go To 99
End If
!
!				Get the Stokes Q columm.
!
If (Present(QPol)) Then
  Call FetchColumn (Unit, npix, 'Q_POLARISATION', QPol, Status)
  If (Status .NE. 0) Go To 99
End If
!
!				Get the Stokes U columm.
!
If (Present(UPol)) Then
  Call FetchColumn (Unit, npix, 'U_POLARISATION', UPol, Status)
  If (Status .NE. 0) Go To 99
End If
!
!				Get the bandpass mismatch, or spurious 
!				signal, columm.
!
If (Present(SpSig)) Then
  Call FetchColumn (Unit, npix, 'SPUR_SIGNAL', SpSig, Status)
  If (Status .NE. 0) Go To 99
End If
!
!				Get the N_Obs columm.
!
If (Present(NObs)) Then
  Call FetchColumn (Unit, npix, 'N_Obs', NObs, Status)
  If (Status .NE. 0) Go To 99
End If
!
!			Extract data from this first extension table.
!
If (Present(Covar)) Then
  Call FTMAHD (unit, 3, tmp, Status)			! Second ext. HDU. 
  If (Status .NE. 0) Go To 99
!
!				Get the covariance matrix.
!
  Call FTGCNO (Unit, .FALSE., 'M11', tmp, Status)
  If (Status .NE. 0) Then
    Status = 0
    Call FetchCovar2x2 (unit, npix, Covar, Status)
  Else
    Call FetchCovar3x3 (unit, npix, Covar, Status)
  End If
  If (Status .NE. 0) Go To 99
!
!			Done!  Set the number of pixels, close the FITS file
!			and return.
!
End If
If (Present(NPixels)) NPixels = npix
!
99 Continue
!
lst = 0
Call FTCLOS (unit, lst)					! Close the file.
Call FTFIOU (unit, lst)					! Release the unit #.
If ((lst .NE. 0) .AND. (Status .EQ. 0)) Status = lst
!
Return
! ----------------------------------------------------------------------------
Contains
! ============================================================================
! "FetchColumn" extracts a column from the current HDU, allocating space for
! it as necessary.
!
! Arguments:
!	Unit   - The open FITS file.
!	NPix   - The number of pixels in a map.
!	Cname  - The name of the column to extract.
!	Dst    - The array to store the column in.  This will be allocated if
!	         it is not associated.
!	Status - A status code:  0=success.
!
! Written by Michael R. Greason, SSAI, 13 March 2006.
! ============================================================================
Subroutine FetchColumn (Unit, NPix, Cname, Dst, Status)
!
Integer (Kind=4),           Intent(In)  :: Unit
Integer (Kind=4),           Intent(In)  :: NPix
Character (*),              Intent(In)  :: Cname
Real (Kind=4),Dimension(:), Pointer     :: Dst
Integer (Kind=4),           Intent(Out) :: Status
!
Integer (Kind=4) :: colnum
Logical          :: anyf
! ----------------------------------------------------------------------------
!			Get the column number for the specifed column.
!			If the column does not exist, FITSIO error 219
!			is returned.
!
Call FTGCNO (Unit, .FALSE., Cname, colnum, Status)
If (Status .NE. 0) Return
!
!			Allocate space for the column, if necessary.
!
If (.NOT. Associated(Dst)) Then
  Allocate(Dst(NPix), Stat=Status)
  If (Status .NE. 0) Return
End If
!
!			Read the column.
!
Call FTGCVE (Unit, colnum, 1, 1, NPix, 0.0E0, Dst, anyf, Status)
!
Return
! ----------------------------------------------------------------------------
End Subroutine FetchColumn
! ============================================================================
! "FetchCovar2x2" extracts the 2x2 covariance matrices from the current HDU,
! allocating space for it as necessary.
!
! Arguments:
!	Unit   - The open FITS file.
!	NPix   - The number of pixels in a map.
!	Dst    - The array to store the matrices in.  This will be allocated if
!	         it is not associated.
!	Status - A status code:  0=success.
!
! Written by Michael R. Greason, SSAI, 13 March 2006.
! ============================================================================
Subroutine FetchCovar2x2 (Unit, NPix, Dst, Status)
!
Integer (Kind=4),               Intent(In)  :: Unit
Integer (Kind=4),               Intent(In)  :: NPix
Real (Kind=4),Dimension(:,:,:), Pointer     :: Dst
Integer (Kind=4),               Intent(Out) :: Status
!
Integer (Kind=4) :: colnum
Logical          :: anyf
Real (Kind=4), Dimension(:), Allocatable :: Tmp
! ----------------------------------------------------------------------------
!			Allocate space as necessary.
!
Allocate(Tmp(NPix), Stat=Status)
If (Status .NE. 0) Return
!
If (.NOT. Associated(Dst)) Then
  Allocate(Dst(2, 2, NPix), Stat=Status)
  If (Status .NE. 0) Go To 98
End If
!
!			Read the QQ column.
!
Call FTGCNO (Unit, .FALSE., 'QQ', colnum, Status)
Call FTGCVE (Unit, colnum, 1, 1, NPix, 0.0E0, Tmp, anyf, Status)
If (Status .NE. 0) Go To 98
Dst(1,1,:) = Tmp
!
!			Read the QU column.
!
Call FTGCNO (Unit, .FALSE., 'QU', colnum, Status)
Call FTGCVE (Unit, colnum, 1, 1, NPix, 0.0E0, Tmp, anyf, Status)
If (Status .NE. 0) Go To 98
Dst(1,2,:) = Tmp
Dst(2,1,:) = Tmp
!
!			Read the UU column.
!
Call FTGCNO (Unit, .FALSE., 'UU', colnum, Status)
Call FTGCVE (Unit, colnum, 1, 1, NPix, 0.0E0, Tmp, anyf, Status)
If (Status .NE. 0) Go To 98
Dst(2,2,:) = Tmp
!
98 Continue
Deallocate(Tmp)
!
Return
! ----------------------------------------------------------------------------
End Subroutine FetchCovar2x2
! ============================================================================
! "FetchCovar3x3" extracts the 3x3 covariance matrices from the current HDU,
! allocating space for it as necessary.
!
! Arguments:
!	Unit   - The open FITS file.
!	NPix   - The number of pixels in a map.
!	Dst    - The array to store the matrices in.  This will be allocated if
!	         it is not associated.
!	Status - A status code:  0=success.
!
! Written by Michael R. Greason, SSAI, 13 March 2006.
! ============================================================================
Subroutine FetchCovar3x3 (Unit, NPix, Dst, Status)
!
Integer (Kind=4),               Intent(In)  :: Unit
Integer (Kind=4),               Intent(In)  :: NPix
Real (Kind=4),Dimension(:,:,:), Pointer     :: Dst
Integer (Kind=4),               Intent(Out) :: Status
!
Integer (Kind=4) :: colnum
Logical          :: anyf
Real (Kind=4), Dimension(:), Allocatable :: Tmp
! ----------------------------------------------------------------------------
!			Allocate space as necessary.
!
Allocate(Tmp(NPix), Stat=Status)
If (Status .NE. 0) Return
!
If (.NOT. Associated(Dst)) Then
  Allocate(Dst(3, 3, NPix), Stat=Status)
  If (Status .NE. 0) Go To 97
End If
!
!			Read the M11 column.
!
Call FTGCNO (Unit, .FALSE., 'M11', colnum, Status)
Call FTGCVE (Unit, colnum, 1, 1, NPix, 0.0E0, Tmp, anyf, Status)
If (Status .NE. 0) Go To 97
Dst(1,1,:) = Tmp
!
!			Read the M12 column.
!
Call FTGCNO (Unit, .FALSE., 'M12', colnum, Status)
Call FTGCVE (Unit, colnum, 1, 1, NPix, 0.0E0, Tmp, anyf, Status)
If (Status .NE. 0) Go To 97
Dst(1,2,:) = Tmp
Dst(2,1,:) = Tmp
!
!			Read the M13 column.
!
Call FTGCNO (Unit, .FALSE., 'M13', colnum, Status)
Call FTGCVE (Unit, colnum, 1, 1, NPix, 0.0E0, Tmp, anyf, Status)
If (Status .NE. 0) Go To 97
Dst(1,3,:) = Tmp
Dst(3,1,:) = Tmp
!
!			Read the M22 column.
!
Call FTGCNO (Unit, .FALSE., 'M22', colnum, Status)
Call FTGCVE (Unit, colnum, 1, 1, NPix, 0.0E0, Tmp, anyf, Status)
If (Status .NE. 0) Go To 97
Dst(2,2,:) = Tmp
!
!			Read the M23 column.
!
Call FTGCNO (Unit, .FALSE., 'M23', colnum, Status)
Call FTGCVE (Unit, colnum, 1, 1, NPix, 0.0E0, Tmp, anyf, Status)
If (Status .NE. 0) Go To 97
Dst(2,3,:) = Tmp
Dst(3,2,:) = Tmp
!
!			Read the M33 column.
!
Call FTGCNO (Unit, .FALSE., 'M33', colnum, Status)
Call FTGCVE (Unit, colnum, 1, 1, NPix, 0.0E0, Tmp, anyf, Status)
If (Status .NE. 0) Go To 97
Dst(3,3,:) = Tmp
!
97 Continue
Deallocate(Tmp)
! 
Return
! ----------------------------------------------------------------------------
End Subroutine FetchCovar3x3
! ============================================================================
End Subroutine WMAP_Read_Map
