; ==============================================================================
; 'WMAP_Read_Map' reads a WMAP skymap FITS file.  The maps are returned through
; various keyword arguments, allowing the user to select the elements that
; s/he desires.
;
; This procedure requires the IDL Astronomy Users Library, available from
; http://idlastro.gsfc.nasa.gov/
;
; Input Arguments:
;	File    - The name of the FITS file.
;
; Output Arguments:
;	Info    - This is a structure that describes the contents of the file,
;	          regardless of what was returned.  The flags, with the
;	          exception of the covariance matrices flag, simply indicate
;	          the presence of the map:  0=no, 1=yes.
;			NPixels     - The number of pixels in the maps.
;			Temperature - A temperature map flag.
;			Q_Pol       - A Stokes Q map flag.
;			U_Pol       - A Stokes U map flag.
;			SpurSig     - A bandpass mismatch/spurious signal map flag.
;			N_Obs       - A N_Obs array flag.
;			Covar       - A covariance matrices flag: 0=not available,
;			              2=2x2 matrices are available,
;			              3=3x3 matrices are available.
;
; Output Keyword Arguments:
;	Temp    - The Stokes I/temperature map.
;	QPol    - The Stokes Q map.
;	UPol    - The Stokes U map.
;	SpSig   - The bandpass mismatch/spurious signal map.
;	NObs    - The N_Obs array.
;	Covar   - The polarization covariance matrices.  This is either a
;	          2x2 or 3x3 matrix for each pixel, depending upon the 
;	          presence of a spurious signal map.
;	NPixels - The number of pixels in the maps.
;
; Writte by Michael R. Greason, SSAI, 13 March 2006.
; ==============================================================================
Pro WMAP_Read_Map, File, Info, Temp=temp, QPol=qpol, UPol=upol,  	$
                   SpSig=spsig, NObs=nobs, Covar=covar, NPixels=npix
;
on_error, 2
;
;			Check arguments.
;
If (n_params() LT 1) Then message, 'Syntax:  WMAP_Read_Map, File [, Info]'
;
;			Read the tables from the FITS File.
;
fits_info, File, /Silent, N_ext=n_ext
If (n_ext LE 0) Then message, strtrim(File,2), ' is not a WMAP skymap file!'
;
tabt = mrdfits(File, 1, /Silent)
If (n_ext GT 1) Then tabc = mrdfits(File, 2, /Silent)
;
npix  = n_elements(tabt)
Info = {WMAPInf, 							$
	NPixels:npix, 							$
	Temperature:0, 							$
	Q_Pol:0, 							$
	U_Pol:0, 							$
	SpurSig:0, 							$
	N_Obs:0,							$
	Covar:0}
;
;			Process the first table.
;
;				Inventory what is available.
;
tn = tag_names(tabt)
tp = 0
qf = 0
uf = 0
sp = 0
nb = 0
;
For i = 0, (n_elements(tn) - 1) Do Begin
  Case (strupcase(tn[i])) Of
    'TEMPERATURE'    : tp = 1
    'Q_POLARISATION' : qf = 1
    'U_POLARISATION' : uf = 1
    'SPUR_SIGNAL'    : sp = 1
    'N_OBS'	     : nb = 1
    Else             : nb = 0	; Placeholder statement.
  EndCase
EndFor
;
Info.Temperature = tp
Info.Q_Pol       = qf
Info.U_Pol       = uf
Info.SpurSig     = sp
Info.N_Obs       = nb
;
;				Extract the various maps.
;
If (tp EQ 1) Then temp  = tabt.temperature
If (qf EQ 1) Then qpol  = tabt.q_polarisation
If (uf EQ 1) Then upol  = tabt.u_polarisation
If (sp EQ 1) Then spsig = tabt.spur_signal
If (nb EQ 1) Then nobs  = tabt.n_obs
;
If (n_ext LE 1) Then Return
;
;			Process the second table.
;
tn = tag_names(tabc)
Info.Covar = 2
For i = 0, (n_elements(tn) - 1) Do Begin
  If (strupcase(tn[i]) EQ 'M11') Then Info.Covar = 3
EndFor
;
If (Info.Covar EQ 2) Then Begin
;
;				Extract 2x2 covariance matrices.
;
  covar = fltarr(2,2,npix)
  covar[0,0,*] = tabc.QQ
  covar[0,1,*] = tabc.QU
  covar[1,0,*] = tabc.QU
  covar[1,1,*] = tabc.UU
;
EndIf Else Begin
;
;				Extract 3x3 covariance matrices.
;
  covar = fltarr(3,3,npix)
  covar[0,0,*] = tabc.M11
  covar[0,1,*] = tabc.M12
  covar[1,0,*] = tabc.M12
  covar[0,2,*] = tabc.M13
  covar[2,0,*] = tabc.M13
  covar[1,1,*] = tabc.M22
  covar[1,2,*] = tabc.M23
  covar[2,1,*] = tabc.M23
  covar[2,2,*] = tabc.M33
;
EndElse
;
Return
End
