! ============================================================================
! "WMAP_Read_Ninv_Weighted_Map" reads a WMAP map file containing Stokes I, Q,
! and U maps plus the bandpass mismatch 'spurious signal' map.  An N-inverse
! covariance matrix is then multiplied into the maps.
!
! The number of maps read from the map file is determined by the size of the
! N-inverse matrix:
! - If the matrix is 4Nx4N, where N is the number of pixels in a map, then the
!   Stokes I, Q, and U maps plus the bandpass mismatch 'spurious signal' map
!   are read, lined up in the order I,Q,U,S, and this vector is multiplied
!   into the matrix.
! - If the matrix is 2Nx2N, then the Stokes Q and U maps are read, ordered Q,U,
!   and multiplied into the matrix; any Stokes I and spurious signal maps are
!   ignored.
!
! The "WMAP_Read_Map" and "WMAP_Read_NInv" subroutines are used to read the
! maps; this subroutine calls them and performs the matrix multiplication.
!
! Arguments:
!	MapFile  - The name of the file containing the maps.
!	NInvFile - The name of the file containing the N-inverse covariance
!	           matrix.
!	Status   - A status code:  0=success.  Most will be FITS I/O or memory
!	           allocation errors.  -1 indicates that the file maps do not
!	           have NPix pixels.
!	Map      - The NPixels x NMaps array to store the final maps in.  If
!	           NMaps is 4 then the maps will be in the order I, Q, U, S;
!	           if NMaps is 2 then the map order will be Q, U.
!	NPixels  - The number of pixels in each map.  Optional.
!	NMaps    - The number of maps in Map.  Optional.
!
! If the Map array is unassociated then it will be allocated by this
! subroutine.  If it is associated then it is assumed to be large enough!
!
! Written by Michael R. Greason, SSAI, 23 March 2006.
! ============================================================================
Subroutine WMAP_Read_Ninv_Weighted_Map (MapFile, NinvFile, Status, 	&
                                        Map, NPixels, NMaps)
!
Implicit None
!
Character (*),                 Intent(In)  :: MapFile
Character (*),                 Intent(In)  :: NinvFile
Integer(Kind=4),               Intent(Out) :: Status
Real (Kind=4), Dimension(:,:), Pointer     :: Map
Integer (Kind=4), Optional,    Intent(Out) :: NPixels
Integer (Kind=4), Optional,    Intent(Out) :: NMaps
!
Integer (Kind=4) :: NP, NM, NEle
Logical          :: AllocFlg
Real (Kind=4), Dimension(:,:), Pointer :: NInv
!
Include 'wmap_read_ninv.fh'
! ----------------------------------------------------------------------------
Status   = 0
NP       = 0
NM       = 0
AllocFlg = .FALSE.
!
!		Read the N-Inverse matrix first.  This will identify the
!		maps are needed from the map file.
!
Call WMAP_Read_NInv (NinvFile, Status, NInv, NEle, NP)
If (Status .EQ. 0) Then
!
!		Read and process the maps.
!
  If (NEle .LT. (4 * NP)) Then
!
!			If NEle is 2NP, only Stokes Q and U  maps are
!			needed.
!
    Call WMAP_Read_Ninv_2x2 (MapFile, NInv, NP, Map, Status, AllocFlg)
    NM = 2
!
  Else
!
!			If NEle is 4NP, the Stokes I, Q, U and the
!			spurious signal maps are needed.
!
    Call WMAP_Read_Ninv_4x4 (MapFile, NInv, NP, Map, Status, AllocFlg)
    NM = 4
!
  End If
!
!		Finish up and return.
!
  If (Status .EQ. 0) Then
    If (Present(NPixels)) NPixels = NP
    If (Present(NMaps))   NMaps   = NM
  End If
!
End If
!
If ((Status .NE. 0) .AND. (AllocFlg) .AND. (Associated(Map))) Deallocate(Map)
!
If (Associated(NInv)) Deallocate(NInv)
!
Return
! ----------------------------------------------------------------------------
Contains
! ============================================================================
! "WMAP_Read_Ninv_2x2" reads a WMAP map file containing Stokes Q and U maps.
! An N-inverse covariance matrix is then multiplied into the maps.  Each row
! of the covariance matrix has the order Q,U where there are NPix elements
! in the Q and U sections of the row.
!
! Arguments:
!	File   - The name of the file containing the maps.
!	Ninv   - The N-inverse covariance matrix.
!	NPix   - The number of pixels in each map.  This is supplied (from
!	         the Ninv file) to this subroutine to be compared to the
!	         number of pixels in the maps in the file.
!	Map    - The NPix x 2 array to store the final maps in.  The second
!	         dimension will have the order: Q, U
!	Status - A status code:  0=success.  Most will be FITS I/O or memory
!	         allocation errors.  -1 indicates that the file maps do not
!	         have NPix pixels.
!	AFlg   - A logical flag indicating whether or not this routine needed
!	         to allocate the output array.
!
! If the Map array is unassociated then it will be allocated by this
! subroutine.  If it is associated then it is assumed to be large enough!
!
! Written by Michael R. Greason, SSAI, 23 March 2006.
! ============================================================================
Subroutine WMAP_Read_Ninv_2x2 (File, Ninv, NPix, Map, Status, AFlg)
!
Character (*),                 Intent(In)  :: File
Real (Kind=4), Dimension(:,:), Pointer     :: NInv
Integer (Kind=4),              Intent(In)  :: NPix
Real (Kind=4), Dimension(:,:), Pointer     :: Map
Integer (Kind=4),              Intent(Out) :: Status
Logical,                       Intent(Out) :: AFlg
!
Integer (Kind=4) :: Inmap, Jnmap, Isrc, Ininv, Jninv, Limit
Real (Kind=4), Dimension(:),   Pointer :: QMap, UMap, SrcMap
!
Integer (Kind=4), Parameter :: NMP = 2
!
Include 'wmap_read_map.fh'
! ----------------------------------------------------------------------------
Status = 0
!
!		Read the required maps.  If the number of pixels in the maps
!		do not match the expected number of pixels then return with
!		Status=-1.
!
Call WMAP_Read_Map (File, Status, QPol=QMap, UPol=UMap, NPixels=Inmap)
If (Status .NE. 0) Go To 98
If (Inmap .NE. NPix) Then
  Status = -1
  Go To 98
End If
!
!		Allocate the output maps if necessary.  Return on an
!		allocation error.
!
AFlg = .FALSE.
If (.NOT. Associated(Map)) Then
  Allocate(Map(NPix, NMP), Stat=Status)
  If (Status .NE. 0) Go To 98
  AFlg = .TRUE.
End If
Map = 0.0
!
!		Populate the output array, performing a matrix multiplication
!		of the input maps with the N-inverse matrix.  Fortunately the
!		N-inverse matrix is symmetric so we needn't worry about
!		row/column issues.
!
Inmap = 1
Jnmap = 1
Limit = NMP * Npix
Do Jninv = 1, Limit
  If (Inmap .GT. NPix) Then
    Inmap = 1
    Jnmap = Jnmap + 1
  End If
  SrcMap => QMap
  Isrc   =  1
  Do Ininv = 1, Limit
    If (Isrc .GT. NPix) Then
      SrcMap => UMap
      Isrc   =  1;
    End If
    Map(Inmap,Jnmap) = Real(Real(Map(Inmap,Jnmap), Kind=8)		&
                     + (Real(Ninv(Ininv,Jninv), Kind=8)			&
		     * Real(SrcMap(Isrc), Kind=8)), Kind=4)
    Isrc = Isrc + 1
  End Do
  Inmap = Inmap + 1
End Do
!
!		Deallocate temporary space.
!
98 Continue
If (Associated(QMap)) Deallocate(QMap)
If (Associated(UMap)) Deallocate(UMap)
!
Return
! ----------------------------------------------------------------------------
End Subroutine WMAP_Read_Ninv_2x2
! ============================================================================
! "WMAP_Read_Ninv_4x4" reads a WMAP map file containing Stokes I, Q, and U
! maps plus the bandpass mismatch 'spurious signal' map.  An N-inverse
! covariance matrix is then multiplied into the maps.  Each row of the
! covariance matrix has the order I,Q,U,S where there are NPix elements in
! the I, Q, U, and S sections of the row.
!
! Arguments:
!	File   - The name of the file containing the maps.
!	Ninv   - The N-inverse covariance matrix.
!	NPix   - The number of pixels in each map.  This is supplied (from
!	         the Ninv file) to this subroutine to be compared to the
!	         number of pixels in the maps in the file.
!	Map    - The NPix x 4 array to store the final maps in.  The second
!	         dimension will have the order: I, Q, U, S.
!	Status - A status code:  0=success.  Most will be FITS I/O or memory
!	         allocation errors.  -1 indicates that the file maps do not
!	         have NPix pixels.
!	AFlg   - A logical flag indicating whether or not this routine needed
!	         to allocate the output array.
!
! If the Map array is unassociated then it will be allocated by this
! subroutine.  If it is associated then it is assumed to be large enough!
!
! Written by Michael R. Greason, SSAI, 23 March 2006.
! ============================================================================
Subroutine WMAP_Read_Ninv_4x4 (File, Ninv, NPix, Map, Status, AFlg)
!
Character (*),                 Intent(In)  :: File
Real (Kind=4), Dimension(:,:), Pointer     :: NInv
Integer (Kind=4),              Intent(In)  :: NPix
Real (Kind=4), Dimension(:,:), Pointer     :: Map
Integer (Kind=4),              Intent(Out) :: Status
Logical,                       Intent(Out) :: AFlg
!
Integer (Kind=4) :: Inmap, Jnmap, Isrc, Jsrc, Ininv, Jninv, Limit
Real (Kind=4), Dimension(:),   Pointer :: IMap, QMap, UMap, SMap, SrcMap
!
Integer (Kind=4), Parameter :: NMP = 4
!
Include 'wmap_read_map.fh'
! ----------------------------------------------------------------------------
Status = 0
!
!		Read the required maps.  If the number of pixels in the maps
!		do not match the expected number of pixels then return with
!		Status=-1.
!
Call WMAP_Read_Map (File, Status, Temp=IMap, QPol=QMap, UPol=UMap, 	&
                    SpSig=SMap, NPixels=Inmap)
If (Status .NE. 0) Go To 99
If (Inmap .NE. NPix) Then
  Status = -1
  Go To 99
End If
!
!		Allocate the output maps if necessary.  Return on an
!		allocation error.
!
AFlg = .FALSE.
If (.NOT. Associated(Map)) Then
  Allocate(Map(NPix, NMP), Stat=Status)
  If (Status .NE. 0) Go To 99
  AFlg = .TRUE.
End If
Map = 0.0
!
!		Populate the output array, performing a matrix multiplication
!		of the input maps with the N-inverse matrix.  Fortunately the
!		N-inverse matrix is symmetric so we needn't worry about
!		row/column issues.
!
Inmap = 1
Jnmap = 1
Limit = NMP * Npix
Do Jninv = 1, Limit
  If (Inmap .GT. NPix) Then
    Inmap = 1
    Jnmap = Jnmap + 1
  End If
  SrcMap => IMap
  Isrc   =  1
  Jsrc   =  1
  Do Ininv = 1, Limit
    If (Isrc .GT. NPix) Then
      Isrc   =  1;
      Jsrc   = Jsrc + 1
      Select Case (Jsrc)
        Case (4)
	  SrcMap => SMap
        Case (3)
	  SrcMap => UMap
        Case (2)
	  SrcMap => QMap
        Case Default
	  SrcMap => IMap
      End Select
    End If
    Map(Inmap,Jnmap) = Real(Real(Map(Inmap,Jnmap), Kind=8)		&
                     + (Real(Ninv(Ininv,Jninv), Kind=8)			&
		     * Real(SrcMap(Isrc), Kind=8)), Kind=4)
    Isrc = Isrc + 1
  End Do
  Inmap = Inmap + 1
End Do
!
!		Deallocate temporary space.
!
99 Continue
If (Associated(IMap)) Deallocate(IMap)
If (Associated(QMap)) Deallocate(QMap)
If (Associated(UMap)) Deallocate(UMap)
If (Associated(SMap)) Deallocate(SMap)
!
Return
! ----------------------------------------------------------------------------
End Subroutine WMAP_Read_Ninv_4x4
! ============================================================================
End Subroutine WMAP_Read_Ninv_Weighted_Map
