; ==============================================================================
; 'WMAP_Read_NInv_Weighted_Map' reads a WMAP map file containing Stokes I, Q,
; and U maps plus the bandpass mismatch 'spurious signal' map.  An N-inverse
; covariance matrix is then multiplied into the maps.
;
; The number of maps read from the map file is determined by the size of the
; N-inverse matrix:
; - If the matrix is 4Nx4N, where N is the number of pixels in a map, then the
;   Stokes I, Q, and U maps plus the bandpass mismatch 'spurious signal' map
;   are read, lined up in the order I,Q,U,S, and this vector is multiplied
;   into the matrix.
; - If the matrix is 2Nx2N, then the Stokes Q and U maps are read, ordered Q,U,
;   and multiplied into the matrix; any Stokes I and spurious signal maps are
;   ignored.
;
; This can be a lengthy process---the arrays may be quite large and take a
; while to read and multiply together.
;
; The "WMAP_Read_Map" and "WMAP_Read_NInv" procedures are used to read the
; maps; this procedure calls them and performs the matrix multiplication.
;
; Input Arguments:
;	MapFile  - The name of the file containing the maps.
;	NInvFile - The name of the file containing the N-inverse covariance
;	           matrix.
;
; Output Arguments:
;	Map      - The NPixels x #Maps array to store the final maps in.  If
;	           #Maps is 4 then the maps will be in the order I, Q, U, S;
;	           if #Maps is 2 then the map order will be Q, U.
;
; Output Keyword Arguments:
;	MapInfo  - The information flags structure returned by 'WMAP_Read_Map'.
;	NPixels  - The number of pixels in each map.
;
; Written by Michael R. Greason, SSAI, 24 March 2006.
; ==============================================================================
Pro WMAP_Read_NInv_Weighted_Map, MapFile, NinvFile, Maps, 		$
                                 MapInfo=info, NPixels=npix
;
on_error, 2
;
;			Check arguments.
;
If (n_params() LT 3) Then message, 					$
	'Syntax: WMAP_Read_NInv_Weighted_Map, MapFile, NinvFile, Maps'
;
;			Read the N-Inverse matrix first. This will identify
;			the maps needed from the map file.
;
WMAP_Read_Ninv, NinvFile, NInv, NElements=nele, NPixels=npix
;
If (nele LT (4 * npix)) Then Begin
;
;			If nele is 2NP, only Stokes Q and U maps are
;			needed.
;
  WMAP_Read_Map, MapFile, info, QPol=qmap, UPol=umap, NPixels=np
  If (np NE npix) Then message, 						$
	'WMAP_Read_NInv_Weighted_Map:  The maps and n-inverse covar. matrix do not match!'
  tmp = double([qmap, umap])
  Maps = float(double(Ninv) # tmp)
  Maps = reform(Maps, npix, 2, /overwrite)
;
EndIf Else Begin
;
;			If nele is 4NP, Stokes I, Q, U and the spurious signal
;			maps are needed.
;
  WMAP_Read_Map, MapFile, info, Temp=imap, QPol=qmap, UPol=umap, SpSig=smap, NPixels=np
  If (np NE npix) Then message, 						$
	'WMAP_Read_NInv_Weighted_Map:  The maps and n-inverse covar. matrix do not match!'
  tmp = double([imap, qmap, umap, smap])
  Maps = float(double(Ninv) # tmp)
  Maps = reform(Maps, npix, 4, /overwrite)
;
EndElse
;
Return
End
